@@ -34,6 +34,10 @@ db_init(void)
 	// All dates are stored as the number of seconds since 1970.
 	// Note that numeric arguments in parentheses that following the type name
 	// are ignored by SQLite - there's no need to impose any length limits.
+	//
+	// feeds' download_date and update_date are special timestamps:
+	// download_date stores the date of last successful feed download,
+	// update_date stores the date of last feed download attempt.
 	sqlite3_exec(
 		db,
 		"CREATE TABLE IF NOT EXISTS feeds("
@@ -109,7 +113,7 @@ db_vacuum(void)
 }
 
 bool
-query_database_file_optimization(void)
+exec_database_file_optimization(void)
 {
 	if (get_cfg_bool(NULL, CFG_CLEAN_DATABASE_ON_STARTUP) == true) {
 		if (db_vacuum() == false) {
@@ -221,6 +225,7 @@ db_get_date_from_feeds_table(const struct string *url, const char *column, size_
 	db_bind_string(res, 1, url);
 	int64_t date = sqlite3_step(res) == SQLITE_ROW ? sqlite3_column_int64(res, 0) : 0;
 	sqlite3_finalize(res);
+	INFO("%s of %s is %" PRId64, column, url->ptr, date);
 	return date;
 }
 
@@ -251,18 +256,43 @@ db_get_string_from_feed_table(const struct string *url, const char *column, size
 }
 
 void
-db_set_download_date(const struct string *url, int64_t download_date)
+db_update_feed_int64(const struct string *url, const char *column_name, int64_t value, bool only_positive)
 {
-	INFO("Setting download_date of %s to %" PRId64, url->ptr, download_date);
-	sqlite3_stmt *res = db_prepare("UPDATE feeds SET download_date=? WHERE feed_url=?", 50);
-	if (res != NULL) {
-		sqlite3_bind_int64(res, 1, download_date);
-		db_bind_string(res, 2, url);
-		if (sqlite3_step(res) == SQLITE_DONE) {
+	if (only_positive == false || value > 0) {
+		char query[100] = {0};
+		strcpy(query, "UPDATE feeds SET ");
+		strcat(query, column_name);
+		strcat(query, "=? WHERE feed_url=?");
+		INFO("Setting %s of %s to %" PRId64, column_name, url->ptr, value);
+		sqlite3_stmt *res = db_prepare(query, strlen(query) + 1);
+		if (res != NULL) {
+			sqlite3_bind_int64(res, 1, value);
+			db_bind_string(res, 2, url);
+			if (sqlite3_step(res) != SQLITE_DONE) {
+				FAIL("Failed to update %s!", column_name);
+			}
+			sqlite3_finalize(res);
+		}
+	}
+}
+
+void
+db_update_feed_string(const struct string *url, const char *column_name, const struct string *value, bool only_nonempty)
+{
+	if (only_nonempty == false || (value != NULL && value->len > 0)) {
+		char query[100] = {0};
+		strcpy(query, "UPDATE feeds SET ");
+		strcat(query, column_name);
+		strcat(query, "=? WHERE feed_url=?");
+		INFO("Setting %s of %s to %s", column_name, url->ptr, value->ptr);
+		sqlite3_stmt *res = db_prepare(query, strlen(query) + 1);
+		if (res != NULL) {
+			db_bind_string(res, 1, value);
+			db_bind_string(res, 2, url);
+			if (sqlite3_step(res) != SQLITE_DONE) {
+				FAIL("Failed to update %s!", column_name);
+			}
 			sqlite3_finalize(res);
-			return;
 		}
-		sqlite3_finalize(res);
 	}
-	FAIL("Failed to set download_date!");
 }
