@@ -33,8 +33,8 @@ get_item_args(struct menu_state *ctx, size_t index)
 	item_fmt[2].value.s = ctx->items->ptr[index].date_str->ptr;
 	item_fmt[3].value.s = ctx->items->ptr[index].pub_date_str->ptr;
 	item_fmt[4].value.s = ctx->items->ptr[index].url->ptr;
-	item_fmt[5].value.s = ctx->items->ptr[index].title ? ctx->items->ptr[index].title->ptr : "";
-	item_fmt[6].value.s = ctx->items->ptr[index].title ? ctx->items->ptr[index].title->ptr : ctx->items->ptr[index].url->ptr;
+	item_fmt[5].value.s = ctx->items->ptr[index].title->ptr;
+	item_fmt[6].value.s = ctx->items->ptr[index].title->len > 0 ? ctx->items->ptr[index].title->ptr : ctx->items->ptr[index].url->ptr;
 	item_fmt[7].value.s = ctx->items->ptr[index].feed[0]->link->ptr;
 	item_fmt[8].value.s = ctx->items->ptr[index].feed[0]->name ? ctx->items->ptr[index].feed[0]->name->ptr : "";
 	item_fmt[9].value.s = ctx->items->ptr[index].feed[0]->name ? ctx->items->ptr[index].feed[0]->name->ptr : ctx->items->ptr[index].feed[0]->link->ptr;
@@ -125,11 +125,12 @@ items_menu_loop(struct menu_state *m)
 	m->unread_state = &is_item_unread;
 	m->write_action = &list_menu_writer;
 	m->entry_format = get_cfg_wstring(NULL, m->flags & MENU_IS_EXPLORE ? CFG_MENU_EXPLORE_ITEM_ENTRY_FORMAT : CFG_MENU_ITEM_ENTRY_FORMAT);
+	items_age += 1;
 	if (m->is_initialized == false) {
 		m->items_age = items_age;
 		m->items = create_items_list(m->feeds_original, m->feeds_count, -1, m->flags & MENU_USE_SEARCH ? search_mode_text_input : NULL);
 		if (m->items == NULL) {
-			return setup_menu(NULL, NULL, 0, 0); // Error displayed by create_items_list
+			return close_menu(); // Error displayed by create_items_list
 		}
 	}
 	start_menu();
@@ -153,22 +154,22 @@ items_menu_loop(struct menu_state *m)
 			case INPUT_MARK_UNREAD_ALL:   mark_all_items_read(m, false);                            break;
 			case INPUT_MARK_IMPORTANT:    mark_item_important(m, m->view_sel, true);                break;
 			case INPUT_MARK_UNIMPORTANT:  mark_item_important(m, m->view_sel, false);               break;
-			case INPUT_RELOAD:            update_feeds(m->items->ptr[m->view_sel].feed, 1);         break;
-			case INPUT_RELOAD_ALL:        update_feeds(m->feeds_original, m->feeds_count);          break;
+			case INPUT_RELOAD:            queue_updates(m->items->ptr[m->view_sel].feed, 1);        break;
+			case INPUT_RELOAD_ALL:        queue_updates(m->feeds_original, m->feeds_count);         break;
 			case INPUT_COPY_TO_CLIPBOARD: copy_string_to_clipboard(m->items->ptr[m->view_sel].url); break;
 			case INPUT_QUIT_HARD:         return NULL;
 			case INPUT_NAVIGATE_BACK:
 				if (get_menu_depth() < 3 && (m->flags & MENU_IS_EXPLORE)) break;
 				// fall through
 			case INPUT_QUIT_SOFT:
-				if (m->flags & MENU_IS_EXPLORE) setup_menu(NULL, NULL, 0, 0);
-				return setup_menu(NULL, NULL, 0, 0);
+				if (m->flags & MENU_IS_EXPLORE) close_menu();
+				return close_menu();
 			case INPUT_TOGGLE_EXPLORE_MODE:
-				if (m->flags & MENU_IS_EXPLORE) return setup_menu(NULL, NULL, 0, 0);
+				if (m->flags & MENU_IS_EXPLORE) return close_menu();
 				break;
 			case INPUT_GOTO_FEED:
-				if (m->flags & MENU_IS_EXPLORE) return setup_menu(&items_menu_loop, m->items->ptr[m->view_sel].feed, 1, MENU_NO_FLAGS);
-				break;
+				if (!(m->flags & MENU_IS_EXPLORE)) break;
+				return setup_menu(&items_menu_loop, m->items->ptr[m->view_sel].feed[0]->name, m->items->ptr[m->view_sel].feed, 1, MENU_NORMAL);
 			case INPUT_APPLY_SEARCH_MODE_FILTER:
 				change_search_filter_of_items_list(&m->items, search_mode_text_input); break;
 			case INPUT_OPEN_IN_BROWSER:
@@ -181,10 +182,10 @@ items_menu_loop(struct menu_state *m)
 			case INPUT_SORT_BY_IMPORTANT:
 				change_items_list_sorting(&m->items, cmd); break;
 			case INPUT_ENTER:
-				return setup_menu(&item_pager_loop, NULL, 0, MENU_NO_FLAGS);
+				return setup_menu(&item_pager_loop, m->items->ptr[m->view_sel].title, NULL, 0, MENU_NORMAL);
 			case INPUT_STATUS_HISTORY_MENU:
-				return setup_menu(&status_pager_loop, NULL, 0, MENU_NO_FLAGS);
+				return setup_menu(&status_pager_loop, NULL, NULL, 0, MENU_NORMAL);
 		}
 	}
-	return setup_menu(NULL, NULL, 0, 0);
+	return close_menu();
 }
