@@ -9,121 +9,32 @@
 #include <pthread.h>
 #include <sqlite3.h>
 #include <curses.h>
+#include <expat.h>
+#include <curl/curl.h>
+#include <yajl/yajl_parse.h>
 
 #ifndef NEWSRAFT_VERSION
-#define NEWSRAFT_VERSION "0.24"
+#define NEWSRAFT_VERSION "0.26"
 #endif
 
 #define ISWHITESPACE(A) (((A)==' ')||((A)=='\n')||((A)=='\t')||((A)=='\v')||((A)=='\f')||((A)=='\r'))
 #define ISWHITESPACEEXCEPTNEWLINE(A) (((A)==' ')||((A)=='\t')||((A)=='\v')||((A)=='\f')||((A)=='\r'))
 #define ISWIDEWHITESPACE(A) (((A)==L' ')||((A)==L'\n')||((A)==L'\t')||((A)==L'\v')||((A)==L'\f')||((A)==L'\r'))
 #define ISDIGIT(A) (((A)=='0')||((A)=='1')||((A)=='2')||((A)=='3')||((A)=='4')||((A)=='5')||((A)=='6')||((A)=='7')||((A)=='8')||((A)=='9'))
-#define INFO(...) do { if (log_stream) { fputs("[INFO] ", log_stream); fprintf(log_stream, __VA_ARGS__); fputc('\n', log_stream); } } while (0)
-#define WARN(...) do { if (log_stream) { fputs("[WARN] ", log_stream); fprintf(log_stream, __VA_ARGS__); fputc('\n', log_stream); } } while (0)
-#define FAIL(...) do { if (log_stream) { fputs("[FAIL] ", log_stream); fprintf(log_stream, __VA_ARGS__); fputc('\n', log_stream); } } while (0)
-#define good_status(...) status_write(CFG_COLOR_STATUS_GOOD, __VA_ARGS__)
+#define INFO(...) log_write("INFO", __VA_ARGS__)
+#define WARN(...) log_write("WARN", __VA_ARGS__)
+#define FAIL(...) log_write("FAIL", __VA_ARGS__)
 #define info_status(...) status_write(CFG_COLOR_STATUS_INFO, __VA_ARGS__)
 #define fail_status(...) status_write(CFG_COLOR_STATUS_FAIL, __VA_ARGS__)
-#define MIN(A, B) ((A) < (B) ? (A) : (B))
+#define LENGTH(A) ((sizeof(A))/(sizeof(*A)))
+#define NEWSRAFT_MIN(A, B) (((A) < (B)) ? (A) : (B)) // Need prefix because some platforms have their own MIN definition
+#define NEWSRAFT_CURSES(CALL) do { if (curses_is_running()) { CALL; } } while (0) // Make CALL only if Curses is running
 
 typedef uint8_t config_entry_id;
-enum {
-	CFG_COLOR_STATUS_GOOD = 0,
-	CFG_COLOR_STATUS_INFO,
-	CFG_COLOR_STATUS_FAIL,
-	CFG_COLOR_LIST_ITEM,
-	CFG_COLOR_LIST_ITEM_UNREAD,
-	CFG_COLOR_LIST_ITEM_IMPORTANT,
-	CFG_COLOR_LIST_FEED,
-	CFG_COLOR_LIST_FEED_UNREAD,
-	CFG_COLOR_LIST_SECTION,
-	CFG_COLOR_LIST_SECTION_UNREAD,
-	CFG_RELOAD_PERIOD,
-	CFG_ITEM_LIMIT,
-	CFG_SCROLLOFF,
-	CFG_PAGER_WIDTH,
-	CFG_UPDATE_THREADS_COUNT,
-	CFG_DOWNLOAD_TIMEOUT,
-	CFG_DOWNLOAD_SPEED_LIMIT,
-	CFG_STATUS_MESSAGES_COUNT_LIMIT,
-	CFG_COPY_TO_CLIPBOARD_COMMAND,
-	CFG_PROXY,
-	CFG_PROXY_USER,
-	CFG_PROXY_PASSWORD,
-	CFG_GLOBAL_SECTION_NAME,
-	CFG_USER_AGENT,
-	CFG_ITEM_CONTENT_FORMAT,
-	CFG_ITEM_CONTENT_DATE_FORMAT,
-	CFG_ITEM_CONTENT_LINK_FORMAT,
-	CFG_LIST_ENTRY_DATE_FORMAT,
-	CFG_OPEN_IN_BROWSER_COMMAND,
-	CFG_NOTIFICATION_COMMAND,
-	CFG_MENU_SECTION_ENTRY_FORMAT,
-	CFG_MENU_FEED_ENTRY_FORMAT,
-	CFG_MENU_ITEM_ENTRY_FORMAT,
-	CFG_MENU_EXPLORE_ITEM_ENTRY_FORMAT,
-	CFG_MENU_FEED_SORTING,
-	CFG_MENU_ITEM_SORTING,
-	CFG_SECTIONS_MENU_PARAMOUNT_EXPLORE,
-	CFG_FEEDS_MENU_PARAMOUNT_EXPLORE,
-	CFG_MARK_ITEM_UNREAD_ON_CHANGE,
-	CFG_MARK_ITEM_READ_ON_HOVER,
-	CFG_ANALYZE_DATABASE_ON_STARTUP,
-	CFG_CLEAN_DATABASE_ON_STARTUP,
-	CFG_RESPECT_TTL_ELEMENT,
-	CFG_RESPECT_EXPIRES_HEADER,
-	CFG_SEND_IF_NONE_MATCH_HEADER,
-	CFG_SEND_IF_MODIFIED_SINCE_HEADER,
-	CFG_PAGER_CENTERING,
-	CFG_ENTRIES_COUNT,
-};
-
 typedef uint8_t input_id;
-enum {
-	INPUT_SELECT_NEXT = 0,
-	INPUT_SELECT_PREV,
-	INPUT_SELECT_NEXT_PAGE,
-	INPUT_SELECT_PREV_PAGE,
-	INPUT_SELECT_FIRST,
-	INPUT_SELECT_LAST,
-	INPUT_JUMP_TO_NEXT,
-	INPUT_JUMP_TO_PREV,
-	INPUT_JUMP_TO_NEXT_UNREAD,
-	INPUT_JUMP_TO_PREV_UNREAD,
-	INPUT_JUMP_TO_NEXT_IMPORTANT,
-	INPUT_JUMP_TO_PREV_IMPORTANT,
-	INPUT_GOTO_FEED,
-	INPUT_SHIFT_WEST,
-	INPUT_SHIFT_EAST,
-	INPUT_SHIFT_RESET,
-	INPUT_SORT_BY_TIME,
-	INPUT_SORT_BY_UNREAD,
-	INPUT_SORT_BY_ALPHABET,
-	INPUT_SORT_BY_IMPORTANT,
-	INPUT_ENTER,
-	INPUT_RELOAD,
-	INPUT_RELOAD_ALL,
-	INPUT_MARK_READ,
-	INPUT_MARK_UNREAD,
-	INPUT_MARK_READ_ALL,
-	INPUT_MARK_UNREAD_ALL,
-	INPUT_MARK_IMPORTANT,
-	INPUT_MARK_UNIMPORTANT,
-	INPUT_TOGGLE_EXPLORE_MODE,
-	INPUT_STATUS_HISTORY_MENU,
-	INPUT_OPEN_IN_BROWSER,
-	INPUT_COPY_TO_CLIPBOARD,
-	INPUT_START_SEARCH_INPUT,
-	INPUT_NAVIGATE_BACK,
-	INPUT_QUIT_SOFT,
-	INPUT_QUIT_HARD,
-	INPUT_SYSTEM_COMMAND,
-	INPUT_ERROR,
-	INPUT_APPLY_SEARCH_MODE_FILTER,
-};
+typedef int newsraft_video_t;
 
-typedef int8_t feeds_column_id;
-enum feed_column {
+enum {
 	FEED_COLUMN_FEED_URL,
 	FEED_COLUMN_TITLE,
 	FEED_COLUMN_LINK,
@@ -140,7 +51,6 @@ enum feed_column {
 	FEED_COLUMN_NONE,
 };
 
-typedef int8_t items_column_id;
 enum {
 	ITEM_COLUMN_FEED_URL,
 	ITEM_COLUMN_GUID,
@@ -166,10 +76,10 @@ enum {
 };
 
 enum {
-	MENU_NO_FLAGS         = 0,
-	MENU_IS_EXPLORE       = 1, // Tell items menu to use explore mode
-	MENU_USE_SEARCH       = 2, // Tell items menu to apply search
-	MENU_SWALLOW          = 4, // Replace current menu with this menu
+	MENU_NORMAL           = 0,
+	MENU_IS_EXPLORE       = 1,  // Tell items menu to use explore mode
+	MENU_USE_SEARCH       = 2,  // Tell items menu to apply search
+	MENU_SWALLOW          = 4,  // Replace current menu with this menu
 	MENU_DISABLE_SETTINGS = 8,
 };
 
@@ -190,19 +100,27 @@ enum { // Even is ascending, odd is descending
 typedef uint8_t render_block_format;
 enum {
 	TEXT_PLAIN,
-	TEXT_RAW, // Same thing as TEXT_PLAIN, but without search for links.
+	TEXT_RAW,   // Same thing as TEXT_PLAIN, but without link marks
 	TEXT_HTML,
+	TEXT_LINKS, // Special block type which has to be populated with links
 };
 
-typedef uint8_t format_hint_mask;
+// Unknown type must have 0 value!
 enum {
-	FORMAT_BOLD_BEGIN = 1,
-	FORMAT_BOLD_END = 2,
-	FORMAT_ITALIC_BEGIN = 4,
-	FORMAT_ITALIC_END = 8,
-	FORMAT_UNDERLINED_BEGIN = 16,
-	FORMAT_UNDERLINED_END = 32,
-	FORMAT_ALL_END = 42, // Sum of all ending hints.
+	MEDIA_TYPE_UNKNOWN = 0,
+	MEDIA_TYPE_XML,
+	MEDIA_TYPE_JSON,
+};
+
+enum {
+	NEWSRAFT_THREAD_DOWNLOAD = 0, // Downloads the feed from the web
+	NEWSRAFT_THREAD_SHRUNNER = 1, // Reads the feed from the command
+	NEWSRAFT_THREAD_DBWRITER = 2, // Writes the feed to the database
+};
+
+enum {
+	NEWSRAFT_HTTP_NOT_MODIFIED      = 304,
+	NEWSRAFT_HTTP_TOO_MANY_REQUESTS = 429,
 };
 
 struct config_context;
@@ -236,10 +154,9 @@ struct feed_entry {
 	struct string *name;
 	struct string *link;
 	int64_t unread_count;
-	int64_t download_date;
+	int64_t update_date; // Date of last feed update attempt
 	struct config_context *cfg;
 	struct input_binding *binds;
-	volatile bool *volatile did_update_just_finished;
 };
 
 struct item_entry {
@@ -277,6 +194,7 @@ struct format_arg {
 };
 
 struct menu_state {
+	struct string *name;
 	struct menu_state *(*run)(struct menu_state *); // Function used to start menu
 	struct feed_entry **feeds_original; // Remains unchanged to use original order
 	struct feed_entry **feeds;          // Virtual feeds with user sorting applied
@@ -304,29 +222,6 @@ struct render_block {
 	bool needs_trimming;
 };
 
-struct render_blocks_list {
-	struct render_block *ptr;
-	size_t len;
-	size_t links_block_index;
-};
-
-struct format_hint {
-	format_hint_mask mask;
-	size_t pos;
-};
-
-struct render_line {
-	struct wstring *ws;
-	struct format_hint *hints;
-	size_t hints_len;
-	size_t indent;
-};
-
-struct render_result {
-	struct render_line *lines;
-	size_t lines_len;
-};
-
 struct link {
 	struct string *url;      // URL link to data.
 	struct string *type;     // Standard MIME type of data.
@@ -339,6 +234,84 @@ struct links_list {
 	size_t len;
 };
 
+struct render_blocks_list {
+	struct render_block *ptr;
+	size_t len;
+	struct links_list links;
+};
+
+struct render_line {
+	struct wstring *ws;
+	newsraft_video_t *hints;
+	size_t hints_len;
+	size_t indent;
+};
+
+struct render_result {
+	struct render_line *lines;
+	size_t lines_len;
+};
+
+struct getfeed_item {
+	struct string *guid;
+	struct string *title;
+	struct string *url;
+	struct string *content;
+	struct string *attachments;
+	struct string *persons;
+	struct string *extras;
+	int64_t publication_date; // Publication date in seconds since the Epoch (0 means unset).
+	int64_t update_date; // Update date in seconds since the Epoch (0 means unset).
+	bool guid_is_url;
+	struct getfeed_item *next;
+};
+
+struct getfeed_feed {
+	struct string *title;
+	struct string *url;
+	struct string *content;
+	struct string *attachments;
+	struct string *persons;
+	struct string *extras;
+	int64_t time_to_live;
+	struct string *http_header_etag;
+	int64_t http_header_last_modified;
+	int64_t http_header_expires;
+	struct getfeed_item *item;
+};
+
+struct feed_update_state {
+	struct feed_entry *feed_entry;
+
+	CURL *curl;
+	char curl_error[CURL_ERROR_SIZE];
+	struct curl_slist *download_headers;
+	long http_response_code;
+
+	int8_t media_type;
+	XML_Parser xml_parser;
+	yajl_handle json_parser;
+	struct getfeed_feed feed;
+	bool in_item;
+	struct string *text;
+	uint8_t path[256];
+	uint8_t depth;
+	struct string decoy;
+	struct string *emptying_target;
+
+	size_t new_items_count;
+
+	bool curl_handle_added_to_multi;
+
+	bool is_in_progress;
+	bool is_downloaded;
+	bool is_failed;
+	bool is_canceled;
+	bool is_finished;
+
+	struct feed_update_state *next;
+};
+
 // See "sections.c" file for implementation.
 int64_t make_sure_section_exists(const struct string *section_name);
 struct feed_entry *copy_feed_to_section(const struct feed_entry *feed_data, int64_t section_index);
@@ -346,6 +319,8 @@ void refresh_unread_items_count_of_all_sections(void);
 bool purge_abandoned_feeds(void);
 struct menu_state *sections_menu_loop(struct menu_state *m);
 void free_sections(void);
+bool start_updating_all_feeds_and_wait_finish(void);
+bool print_unread_items_count(void);
 void process_auto_updating_feeds(void);
 void mark_feeds_read(struct feed_entry **feeds, size_t feeds_count, bool status);
 #ifdef TEST
@@ -371,8 +346,10 @@ bool handle_list_menu_control(struct menu_state *m, input_id cmd, const struct w
 bool handle_pager_menu_control(input_id cmd);
 void free_menus(void);
 size_t get_menu_depth(void);
-struct menu_state *setup_menu(struct menu_state *(*run)(struct menu_state *), struct feed_entry **feeds, size_t feeds_count, uint32_t flags);
+struct menu_state *setup_menu(struct menu_state *(*run)(struct menu_state *), struct string *name, struct feed_entry **feeds, size_t feeds_count, uint32_t flags);
+struct menu_state *close_menu(void);
 void start_menu(void);
+void write_menu_path_string(struct string *names, struct menu_state *m);
 
 // See "interface-list-pager.c" file for implementation.
 bool is_pager_pos_valid(struct menu_state *ctx, size_t index);
@@ -383,6 +360,10 @@ bool refresh_pager_menu(void);
 // See "format.c" file for implementation.
 void do_format(struct wstring *dest, const wchar_t *fmt, const struct format_arg *args);
 
+// See "sorting.c" file for implementation.
+int get_sorting_id(const char *sorting_name);
+const char *get_sorting_message(int sorting_id);
+
 // See "items.c" file for implementation.
 bool important_item_condition(struct menu_state *ctx, size_t index);
 void tell_items_menu_to_regenerate(void);
@@ -405,16 +386,9 @@ struct menu_state *item_pager_loop(struct menu_state *m);
 // and generates a single plain text buffer for a pager to display.
 // See "render-block.c" file for implementation.
 bool add_render_block(struct render_blocks_list *blocks, const char *content, size_t content_len, render_block_format content_type, bool needs_trimming);
-bool apply_links_render_blocks(struct render_blocks_list *blocks, const struct wstring *data);
+void apply_links_render_blocks(struct render_blocks_list *blocks, const struct wstring *data);
 void free_render_blocks(struct render_blocks_list *blocks);
-
-// Here we extract links from texts of render_block entries into links_list and
-// insert link marks into texts so that it's more convenient for user to work
-// with the list of links in the pager. Also, we do here some screen-independent
-// processing of texts that render_block entries have, for example expand a few
-// inline HTML elements like <span>, <sup>, <q>, etc.
-// See "prepare_to_render_data" directory for implementation.
-bool prepare_to_render_data(struct render_blocks_list *blocks, struct links_list *links);
+void free_render_result(struct render_result *render);
 
 // See "render_data" directory for implementation.
 bool render_data(struct config_context **ctx, struct render_result *result, struct render_blocks_list *blocks, size_t content_width);
@@ -427,11 +401,10 @@ bool get_largest_piece_from_item_content(const char *content, struct string **te
 bool get_largest_piece_from_item_attachments(const char *attachments, struct string **text, render_block_format *type);
 
 // See "items-metadata-links.c" file for implementation.
-int64_t add_another_url_to_trim_links_list(struct links_list *links, const char *url, size_t url_len);
-bool populate_link_list_with_links_of_item(struct links_list *links, sqlite3_stmt *res);
+char *complete_url(const char *base, const char *rel);
+int64_t add_url_to_links_list(struct links_list *links, const char *url, size_t url_len);
 struct wstring *generate_link_list_wstring_for_pager(struct config_context **ctx, const struct links_list *links);
-bool complete_urls_of_links(struct links_list *links);
-void free_links_list(const struct links_list *links);
+bool add_item_attachments_to_links_list(struct links_list *links, sqlite3_stmt *res);
 
 // See "items-metadata-persons.c" file for implementation.
 struct string *deserialize_persons_string(const char *src);
@@ -445,13 +418,13 @@ const char *get_config_path(void);
 const char *get_db_path(void);
 
 // See "dates.c" file for implementation.
-int64_t parse_date_rfc3339(const char *src);
+int64_t parse_date(const char *str, bool rfc3339_first);
 struct string *get_cfg_date(struct config_context **ctx, config_entry_id format_id, int64_t date);
 
 // See "db.c" file for implementation.
 bool db_init(void);
 bool db_vacuum(void);
-bool query_database_file_optimization(void);
+bool exec_database_file_optimization(void);
 void db_stop(void);
 sqlite3_stmt *db_prepare(const char *zSql, int nByte);
 bool db_begin_transaction(void);
@@ -461,43 +434,42 @@ const char *db_error_string(void);
 int db_bind_string(sqlite3_stmt *stmt, int pos, const struct string *str);
 int64_t db_get_date_from_feeds_table(const struct string *url, const char *column, size_t column_len);
 struct string *db_get_string_from_feed_table(const struct string *url, const char *column, size_t column_len);
-void db_set_download_date(const struct string *url, int64_t download_date);
+void db_update_feed_int64(const struct string *url, const char *column_name, int64_t value, bool only_positive);
+void db_update_feed_string(const struct string *url, const char *column_name, const struct string *value, bool only_nonempty);
 
 // See "db-items.c" file for implementation.
 sqlite3_stmt *db_find_item_by_rowid(int64_t rowid);
 bool db_mark_item_read(int64_t rowid, bool status);
 bool db_mark_item_important(int64_t rowid, bool status);
 int64_t get_unread_items_count_of_the_feed(const struct string *url);
-int64_t get_items_count_of_feeds(struct feed_entry **feeds, size_t feeds_count);
+int64_t db_count_items(struct feed_entry **feeds, size_t feeds_count, bool count_only_unread);
 bool db_change_unread_status_of_all_items_in_feeds(struct feed_entry **feeds, size_t feeds_count, bool unread);
 
 // See "interface.c" file for implementation.
+bool curses_is_running(void);
 bool curses_init(void);
+void curses_stop(void);
+bool run_menu_loop(void);
 input_id resize_handler(void);
 bool call_resize_handler_if_current_list_menu_size_is_different_from_actual(void);
 bool arent_we_colorful(void);
 
-// Functions related to window which reads user input and displays command
-// counter (prefix number before command).
-// See "interface-input.c" file for implementation.
-bool counter_recreate_unprotected(void);
-void tell_program_to_terminate_safely_and_quickly(int dummy);
-input_id get_input(struct input_binding *ctx, uint32_t *count, const struct wstring **macro_ptr);
-void break_getting_input_command(void);
-void counter_delete(void);
+// See "interface-colors.c" file for implementation.
+int get_color_pair_unprotected(int fg, int bg);
 
 // Functions related to window which displays status messages.
 // See "interface-status.c" file for implementation.
-void update_status_window_content(void);
+void update_status_window_content_unprotected(void);
 bool status_recreate_unprotected(void);
-bool allocate_status_messages_buffer(void);
 void status_clean_unprotected(void);
 void status_clean(void);
 void prevent_status_cleaning(void);
 void allow_status_cleaning(void);
 void status_write(config_entry_id color, const char *format, ...);
-void status_delete(void);
 struct string *generate_string_with_status_messages_for_pager(void);
+void status_delete(void);
+input_id get_input(struct input_binding *ctx, uint32_t *count, const struct wstring **macro_ptr);
+void break_getting_input_command(void);
 
 // See "interface-status-pager.c" file for implementation.
 struct menu_state *status_pager_loop(struct menu_state *dest);
@@ -509,7 +481,9 @@ struct input_binding *create_or_clean_bind(struct input_binding **target, const
 bool attach_action_to_bind(struct input_binding *bind, input_id action);
 bool attach_command_to_bind(struct input_binding *bind, const char *exec, size_t exec_len);
 bool assign_default_binds(void);
+input_id get_input_id_by_name(const char *name);
 void free_binds(struct input_binding *target);
+void free_default_binds(void);
 
 // Functions related to executing system commands.
 // See "commands.c" file for implementation.
@@ -563,7 +537,6 @@ bool register_signal_handlers(void);
 // See "load_config" directory for implementation.
 bool process_config_line(struct feed_entry *feed, const char *str, size_t len);
 bool parse_config_file(void);
-bool load_config(void);
 bool get_cfg_bool(struct config_context **ctx, config_entry_id id);
 size_t get_cfg_uint(struct config_context **ctx, config_entry_id id);
 unsigned int get_cfg_color(struct config_context **ctx, config_entry_id id);
@@ -573,15 +546,19 @@ void free_config(void);
 void free_config_context(struct config_context *cfg);
 
 // Download, process and store new items of feed.
-// See "update_feed" directory for implementation.
-void update_feeds(struct feed_entry **feeds, size_t feeds_count);
-bool start_feed_updater(void);
-bool try_to_stop_feed_updater(void);
+// See "queue.c" file for implementation.
+void queue_updates(struct feed_entry **feeds, size_t feeds_count);
+void queue_wait_finish(void);
+struct feed_update_state *queue_pull(bool (*condition)(struct feed_update_state *));
+void queue_destroy(void);
+void queue_examine(void);
 
 // Functions for opening and closing the log stream.
 // To write to the log stream use macros INFO, WARN or FAIL.
 // See "log.c" file for implementation.
 bool log_init(const char *path);
+void log_write(const char *prefix, const char *format, ...);
+FILE *log_get_stream(void);
 void log_stop(int error_code);
 
 // Functions for buffering errors to prevent
@@ -590,11 +567,45 @@ void log_stop(int error_code);
 void write_error(const char *format, ...);
 void flush_errors(void);
 
-extern volatile bool they_want_us_to_terminate;
-extern FILE *log_stream;
+// See "downloader.c" file for implementation.
+void *downloader_worker(void *dummy);
+bool curl_init(void);
+void curl_stop(void);
+void downloader_curl_wakeup(void);
+void remove_downloader_handle(struct feed_update_state *data);
+
+// See "executor.c" file for implementation.
+void *executor_worker(void *dummy);
+
+// See "inserter.c" file for implementation.
+void *inserter_worker(void *dummy);
+
+// See "threads.c" file for implementation.
+bool threads_start(void);
+void threads_wake_up(int thread_id);
+void threads_take_a_nap(int thread_id);
+void threads_stop(void);
+
+// See "parse_xml" directory for implementation.
+bool setup_xml_parser(struct feed_update_state *data);
+
+// See "parse_json" directory for implementation.
+bool setup_json_parser(struct feed_update_state *data);
+
+// See "insert_feed" directory for implementation.
+bool insert_feed(struct feed_entry *feed, struct getfeed_feed *feed_data);
+
+// See "struct-item.c" file for implementation.
+bool prepend_item(struct getfeed_item **head_item_ptr);
+void free_item(struct getfeed_item *item);
+
+extern volatile bool they_want_us_to_stop;
 extern size_t list_menu_height;
 extern size_t list_menu_width;
 extern bool search_mode_is_enabled;
 extern struct wstring *search_mode_text_input;
 extern pthread_mutex_t interface_lock;
+
+#include "config.h"
+#include "input.h"
 #endif // NEWSRAFT_H
